package ch.std.genericdto.rest;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.context.SpringBootTest.WebEnvironment;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;

import ch.std.genericdto.dto.RequestDTO;
import ch.std.genericdto.dto.ResponseDTO;
import ch.std.genericdto.rest.CalcController.Calc;
import ch.std.genericdto.rest.EchoController.Echo;

@SpringBootTest(webEnvironment = WebEnvironment.RANDOM_PORT)
public class CalcControllerTests {

	Logger logger = LoggerFactory.getLogger(CalcControllerTests.class);

	@Autowired
	private TestRestTemplate restTemplate;

	@BeforeEach
	public void setup() {
		logger.info("CalcControllerTests.setup");
	}

	@Test
	public void testGetEcho() throws Exception {
		String url = "/rest/calc?action=add&a=1.0&b=2.0";
		ResponseEntity<ResponseDTO<Double>> responseDTO = this.restTemplate.exchange(url, HttpMethod.GET, null, new ParameterizedTypeReference<ResponseDTO<Double>>() {});
		assertNotNull(responseDTO);
		assertNotNull(responseDTO.getBody());
		assertEquals(3.0, responseDTO.getBody().getBody());
	}

	@Test
	public void testPostEcho() throws Exception {
		String url = "/rest/calc";
		RequestDTO<Calc> requestDTO = new RequestDTO<Calc>();
		requestDTO.setBody(new Calc("add", 1.0,2.0));
		HttpEntity<RequestDTO<Calc>> httpEntityRequest = new HttpEntity<>(requestDTO);
		ResponseEntity<ResponseDTO<Double>> responseDTO = this.restTemplate.exchange(url, HttpMethod.POST, httpEntityRequest, new ParameterizedTypeReference<ResponseDTO<Double>>() {});
		assertNotNull(responseDTO);
		assertNotNull(responseDTO.getBody());
		assertEquals(3.0, responseDTO.getBody().getBody());
	}

	@Test
	public void testPostEcho2() throws Exception {
		String url = "/rest/echo/real";
		RequestDTO<Echo> requestDTO = new RequestDTO<Echo>();
		requestDTO.setHeadValue("name", "James");
		requestDTO.setBody(new Echo("James"));
		HttpEntity<RequestDTO<Echo>> httpEntityRequest = new HttpEntity<>(requestDTO);		
		ResponseEntity<ResponseDTO<Echo>> responseDTO = this.restTemplate.exchange(url, HttpMethod.POST, httpEntityRequest, new ParameterizedTypeReference<ResponseDTO<Echo>>() {});
		assertNotNull(responseDTO);
		assertNotNull(responseDTO.getBody());
	}

}
